# xsbt-web-plugin

*Version 0.9*

xsbt-web-plugin is an [sbt](http://www.scala-sbt.org/) extension for building [J2EE](http://en.wikipedia.org/wiki/Java_Servlet) Web applications in Scala and Java.

## Getting started 

The quickest way to get started is to clone the [xwp-template](https://github.com/earldouglas/xwp-template) project, which sets up the necessary directories, files, and configuration for a basic xsbt-web-plugin project.

## Quick reference

First, add xsbt-web-plugin to *project/plugins.sbt*:

```scala
addSbtPlugin("com.earldouglas" % "xsbt-web-plugin" % "0.9.1")
```

For [*.sbt* build definitions](http://www.scala-sbt.org/release/docs/Getting-Started/Basic-Def.html), inject the plugin settings in *build.sbt*:

```scala
seq(webSettings :_*)
```

For [*.scala* build definitions](http://www.scala-sbt.org/release/docs/Getting-Started/Full-Def.html), inject the plugin settings in *Build.scala*:

```scala
import com.earldouglas.xsbtwebplugin.WebPlugin

Project(..., settings = Project.defaultSettings ++ WebPlugin.webSettings)
```

Add a Servlet API to the *provided* configuration:

```scala
libraryDependencies += "javax.servlet" % "javax.servlet-api" % "3.0.1" % "provided"
```

Include either Jetty or Tomcat in the *container* configuration:

*Jetty:*

```scala
libraryDependencies ++= Seq(
  "org.eclipse.jetty" % "jetty-webapp" % "9.1.0.v20131115" % "container",
  "org.eclipse.jetty" % "jetty-plus"   % "9.1.0.v20131115" % "container"
)
```

*Tomcat:*

```scala
libraryDependencies ++= Seq(
  "org.apache.tomcat.embed" % "tomcat-embed-core"         % "7.0.22" % "container",
  "org.apache.tomcat.embed" % "tomcat-embed-logging-juli" % "7.0.22" % "container",
  "org.apache.tomcat.embed" % "tomcat-embed-jasper"       % "7.0.22" % "container"
)
```

## Starting from scratch

Create a new empty project:

```
mkdir xwp-template
cd xwp-template
```

Set up the project structure:

```
mkdir project
mkdir -p src/main/scala
mkdir -p src/main/webapp/WEB-INF
```

Configure sbt:

*project/build.properties*:

```
sbt.version=0.13.0
```

*project/plugins.sbt*:
```
addSbtPlugin("com.earldouglas" % "xsbt-web-plugin" % "0.5.0")
```

*build.sbt*:
```
name := "xwp-template"

organization := "com.earldouglas"

version := "0.1.0-SNAPSHOT"

scalaVersion := "2.10.3"

seq(webSettings :_*)

libraryDependencies += "org.eclipse.jetty" % "jetty-webapp" % "9.1.0.v20131115" % "container"

libraryDependencies += "org.eclipse.jetty" % "jetty-plus" % "9.1.0.v20131115" % "container"

libraryDependencies += "javax.servlet" % "servlet-api" % "2.5" % "provided"
```

Add a servlet:

*src/main/scala/XwpTemplateServlet.scala*:

```scala
package com.earldouglas.xwptemplate

import scala.xml.NodeSeq
import javax.servlet.http.HttpServlet

class XwpTemplateServlet extends HttpServlet {

  import javax.servlet.http.HttpServletRequest
  import javax.servlet.http.HttpServletResponse

  override def doGet(request: HttpServletRequest, response: HttpServletResponse) {

    response.setContentType("text/html")
    response.setCharacterEncoding("UTF-8")

    val responseBody: NodeSeq = <html><body><h1>Hello, world!</h1></body></html>
    response.getWriter.write(responseBody.toString)
  }
}
```

*src/main/webapp/WEB-INF/web.xml*:

```xml
<?xml version="1.0" encoding="UTF-8"?>
<web-app
  xmlns="http://java.sun.com/xml/ns/javaee"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xsi:schemaLocation="http://java.sun.com/xml/ns/javaee http://java.sun.com/xml/ns/javaee/web-app_2_5.xsd"
  version="2.5"
  >

  <servlet>
    <servlet-name>xwp template</servlet-name>
    <servlet-class>com.earldouglas.xwptemplate.XwpTemplateServlet</servlet-class>
  </servlet>

  <servlet-mapping>
    <servlet-name>xwp template</servlet-name>
    <url-pattern>/*</url-pattern>
  </servlet-mapping>

</web-app>
```

## Launching from sbt

From sbt, run the command `container:start`:

```
> container:start
[info] jetty-9.1.0.v20131115
[info] NO JSP Support for /, did not find org.apache.jasper.servlet.JspServlet
[info] Started SelectChannelConnector@0.0.0.0:8080
[success] Total time: 0 s, completed May 27, 2013 11:29:14 AM
>
```

The Web application is now running at http://localhost:8080/.  Take a look with a Web browser, or via curl:

```
$ curl -i localhost:8080
HTTP/1.1 200 OK
Content-Type: text/html; charset=utf-8
Content-Length: 48
Server: Jetty(6.1.22)

<html><body><h1>Hello, world!</h1></body></html>
```

## Deploying to a servlet container

To build a WAR file suitable for deployment, run the command `package` from sbt:

```
> package
[success] Total time: 0 s, completed May 27, 2013 11:31:59 AM
> 
```

The WAR file can be found in *target/scala-2.10/xwp-template_2.10-0.1.0-SNAPSHOT.war*.

## Acknowledgments

Thanks to [Mark Harrah](http://github.com/harrah) for the awesome build tool and initial implementation, and especially for "dual/lazy framework" classloader idea.

Thanks also to [Artyom Olshevskiy](https://github.com/siasia) for creating and maintaining the first several versions of xsbt-web-plugin.

Thanks to the whole community for using xsbt-web-plugin and submitting ideas, questions, and issues to us.

## Artifacts

### *.war* file

To disable publishing of the *.war* file, add the setting:

```scala
packagedArtifacts <<= packagedArtifacts map { as => as.filter(_._1.`type` != "war") }
```

Note that `package` can still be used to create the *.war* file under the project *target/* directory.

#### Massaging the *.war* file

```scala
warPostProcess in Compile <<= (target) map {
  (target) => { 
    () =>
      val webapp = target / "webapp"
      IO.delete(webapp / "test.html")
  }
}
```

### *.jar* file

To enable publishing of the project's *.jar* file, add the setting:

```scala
publishArtifact in (Compile, packageBin) := true
```

## Commands

To start a web application, use `container:start`:

```
> container:start
[info] jetty-7.3.0.v20110203
[info] NO JSP Support for /, did not find org.apache.jasper.servlet.JspServlet
[info] started o.e.j.w.WebAppContext{/,[file:/home/siasia/projects/xsbt-web-plugin/src/main/webapp/]}
[info] Started SelectChannelConnector@0.0.0.0:8080
```

The application is now accesible on localhost, port 8080: [http://localhost:8080](http://localhost:8080)

To stop a running web application, use `container:stop`:

```
> container:stop
[info] stopped o.e.j.w.WebAppContext{/,[file:/home/siasia/projects/xsbt-web-plugin/src/main/webapp/]}
```
    
To reload a running web application, use `container:reload <context-path>`:

```
> container:reload /
```

To automatically reload a web application when source code is changed, use `~;container:start; container:reload <context-path>`:

```
> ~;container:start; container:reload /
```

To build a WAR package, use `package`.  To change the output directory of the WAR file, modify its `artifactPath`:

```scala
artifactPath in (Compile, packageWar) ~= { defaultPath =>
  file("dist") / defaultPath.getName
}
```


## Configuration

Plugin keys are located in `com.earldouglas.xsbtwebplugin.PluginKeys`.

### Container settings

```scala
host in container.Configuration := "192.168.1.4"

port in container.Configuration := 8080

ssl in container.Configuration := Some("192.168.1.4", 8443, "keystore_path", "keystore_password", "key_password")

customConfiguration in container.Configuration := true

configurationFiles in container.Configuration := Seq(file("jetty.xml"))

configurationXml in container.Configuration := <xml />
```

### Web application settings

```scala
webappResources in Compile <+= (sourceDirectory in Runtime)(sd => sd / "static")

scanDirectories in Compile += file("lib")

scanInterval in Compile := 0

env in Compile := Some(file(".") / "conf" / "jetty" / "jetty-env.xml" asFile)

fullClasspath in Runtime in packageWar <+= baseDirectory.map(bd => bd / "extras")

classesAsJar in Compile := true
```


## Content

Web application content belongs in the `src/main/webapp` directory.  For example, `web.xml` should be placed in `src/main/webapp/WEB-INF/web.xml`.

## Deployment

### Tomcat

On Ubuntu, install both *tomcat7* and *tomcat7-admin*:

```bash
sudo apt-get install tomcat7 tomcat7-admin
```

Create a Tomcat user with the role **manager-script** in */etc/tomcat7/tomcat-users.xml*:

```xml
<user username="manager" password="secret" roles="manager-script" />
```

Restart tomcat:

```bash
sudo service tomcat7 restart
```

Now a WAR file can be deployed using the Manager *deploy* command:

```
curl --upload-file myapp.war "http://manager:secret@myhost:8080/manager/text/deploy?path=/myapp&update=true"
```

The application will be available at *myhost:8080/myapp*.

Learn more about Manager commands [here](http://tomcat.apache.org/tomcat-7.0-doc/manager-howto.html).

### Heroku

1. Install the [Heroku Toolbelt](https://toolbelt.heroku.com/)

2. Install the `heroku-deploy` command line plugin:

```bash
heroku plugins:install https://github.com/heroku/heroku-deploy
```
3. Create a WAR file:

```bash
sbt package
```

4 Deploy the WAR file:

```bash
heroku deploy:war --war <path_to_war_file> --app <app_name>
```

See [devcenter.heroku.com](https://devcenter.heroku.com/articles/war-deployment) for more information.

### Google App Engine

See [developers.google.com](https://developers.google.com/appengine/docs/java/tools/uploadinganapp) for more information.

### Multiple contexts

Here is an example of how multiple webapps could be deployed to a single Jetty instance:

```scala
import sbt._

import com.github.siasia._
import WebappPlugin.webappSettings
import Keys._
    
object WebBuild extends Build {
  lazy val container = Container("container")

  lazy val rootSettings = Seq(
    libraryDependencies += "org.mortbay.jetty" % "jetty" % "6.1.22" % "container"
  ) ++ container.deploy(
    "/sub1" -> sub1,
    "/sub2" -> sub2
  )    
      
  lazy val sub1 = Project("sub1", file("sub1")) settings(webappSettings :_*)
  lazy val sub2 = Project("sub2", file("sub2")) settings(webappSettings :_*)
  lazy val root = Project("root", file(".")) settings(rootSettings :_*)
  override def projects = Seq(root, sub1, sub2)
}
```
    
Let's play with it a little

    > container:start
    2011-10-02 07:49:53.917:INFO::Logging to STDERR via org.mortbay.log.StdErrLog
    [info] jetty-6.1.22
    [info] NO JSP Support for /sub1, did not find org.apache.jasper.servlet.JspServlet
    [info] NO JSP Support for /sub2, did not find org.apache.jasper.servlet.JspServlet
    [info] Started SocketConnector@0.0.0.0:8080
    [success] Total time: 1 s, completed 02.10.2011 7:49:54
    > container:stop 
    [success] Total time: 0 s, completed 02.10.2011 7:54:34
    > container:start
    [info] jetty-6.1.22
    [info] NO JSP Support for /sub1, did not find org.apache.jasper.servlet.JspServlet
    [info] NO JSP Support for /sub2, did not find org.apache.jasper.servlet.JspServlet
    [info] Started SocketConnector@0.0.0.0:8080
    [success] Total time: 0 s, completed 02.10.2011 7:54:38
    > container:reload /sub 
    /sub1   /sub2
    > container:reload /sub1
    [info] NO JSP Support for /sub1, did not find org.apache.jasper.servlet.JspServlet
    [success] Total time: 0 s, completed 02.10.2011 7:54:53
    >
    
### Deploy different configurations

Simple examples:

Run whole webapp in Runtime conf:
```scala
webSettings(Runtime)
```

Instruct webapp to put it's deployment info into Runtime:
```scala
inConfig(Runtime)(webappSettings0)
```

Override conf which container takes deployment info from:
```scala
container.deploy(Runtime)
```

More complex:

```scala
import sbt._
import Keys._
import com.github.siasia._
import PluginKeys._
import WebPlugin._
import WebappPlugin._

object WebBuild extends Build {
  def rootSettings = webappSettings ++
  inConfig(Runtime)(webappSettings0) ++
  container.settings ++ Seq(    
    apps <<= (deployment in Compile, deployment in Runtime) map {
      (compile, runtime) => Seq(
        "/compile" -> compile,
        "/runtime" -> runtime
      )
    },
    libraryDependencies += "org.mortbay.jetty" % "jetty" % "6.1.22" % "container"
  )
  
  lazy val root = Project("root", file(".")) settings(rootSettings :_*)
}
```

### Additional containers

```scala
import sbt._
import Keys._
import com.github.siasia._
import PluginKeys._
import WebPlugin._
import WebappPlugin._

object WebBuild extends Build {
  val additional = Container("additional")
  
  def rootSettings: Seq[Setting[_]] =
    container.deploy(
      "/" -> subRef
    ) ++
    additional.deploy(
      "/" -> subRef
    ) ++
    Seq(
      libraryDependencies ++= Seq(
        "org.mortbay.jetty" % "jetty" % "6.1.22" % "container",
        "org.mortbay.jetty" % "jetty" % "6.1.22" % "additional"
      ),
      port in additional.Configuration := 8081
    )

  lazy val sub = Project("sub", file("sub")) settings(webappSettings :_*)
  lazy val subRef: ProjectReference = sub
  lazy val root = Project("root", file(".")) settings(rootSettings :_*)
  override def projects = Seq(root, sub)
}
```

## Differencies from 0.1

1. Existing plugin has been broken down into two separate facilities: container and webapp. Compound plugin is still available for simplicity
1. Webapp is running directly from sources without intermediate exploded war assembly
1. Breakdown into two parts allows to implement deployment of multiple projects into one web server
1. And also multiple web servers in one project
1. Web server runner interface is now little more abstract. This would allow to implement Tomcat support someday

## Miscellaneous

### jetty.xml location

Use jettyConfFiles, like so:

```scala
env in Compile := Some(file(".") / "conf" / "jetty" / "jetty-env.xml" asFile)
```
### Error scenarios

#### Object is not of type class org.eclipse.jetty.server.Server

This error can be expected by those migrating from maven-jetty-plugin to sbt. The relevant part of the error message is:

    java.lang.IllegalArgumentException: Object is not of type class org.eclipse.jetty.server.Server
      at org.eclipse.jetty.xml.XmlConfiguration.configure(XmlConfiguration.java:203)
      at org.eclipse.jetty.plus.webapp.EnvConfiguration.configure(EnvConfiguration.java:118)
      ...

This may be caused by the start of the jetty.xml file. Maven requires a server tag that refers to the above class. xsbt-web-plugin needs the following instead:

```xml
<Configure id="Server" class="org.eclipse.jetty.webapp.WebAppContext">
```

## Running Lift

Add Lift and Jetty to project dependencies:

```scala
libraryDependencies ++= Seq(
  "net.liftweb" %% "lift-webkit" % "2.3" % "compile",
  "org.mortbay.jetty" % "jetty" % "6.1.22" % "container",
  "ch.qos.logback" % "logback-classic" % "0.9.26"
)
```
		
or in case if you want to use Jetty 7:

```scala
libraryDependencies ++= Seq(
  "net.liftweb" %% "lift-webkit" % "2.3" % "compile",
  "org.eclipse.jetty" % "jetty-webapp" % "7.3.0.v20110203" % "container",
  "ch.qos.logback" % "logback-classic" % "0.9.26"
)
```

## Settings

Container settings:

```scala
port in container.Configuration := 8081

ssl in container.Configuration := Some(ssl_port, "path_to_keystore", "keystore_password", "key_password")

customConfiguration in container.Configuration := true

configurationFiles in container.Configuration := Seq(file("jetty.xml"))

configurationXml in container.Configuration := <xml />
```

Webapp settings:

```scala
webappResources in Compile <+= (sourceDirectory in Runtime)(sd => sd / "static")

scanDirectories in Compile += file("lib")

scanInterval in Compile := 0

env in Compile := Some(file(".") / "conf" / "jetty" / "jetty-env.xml" asFile)
```

## SSL support

To use SSL, add the following to your SBT file:

```
ssl in container.Configuration := Some(ssl_port, "path_to_keystore", "keystore_password", "key_password")
```
